// SPDX-License-Identifier: GPL-3.0-only
/*
 *  PolyMC - Minecraft Launcher
 *  Copyright (C) 2022 Sefa Eyeoglu <contact@scrumplex.net>
 *  Copyright (C) 2022 Lenny McLennington <lenny@sneed.church>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 3.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * This file incorporates work covered by the following copyright and
 * permission notice:
 *
 *      Copyright 2013-2021 MultiMC Contributors
 *
 *      Licensed under the Apache License, Version 2.0 (the "License");
 *      you may not use this file except in compliance with the License.
 *      You may obtain a copy of the License at
 *
 *          http://www.apache.org/licenses/LICENSE-2.0
 *
 *      Unless required by applicable law or agreed to in writing, software
 *      distributed under the License is distributed on an "AS IS" BASIS,
 *      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *      See the License for the specific language governing permissions and
 *      limitations under the License.
 */

#include "BuildConfig.h"
#include <QObject>

const Config BuildConfig;

Config::Config()
{
    // Name and copyright
    LAUNCHER_NAME = "PolyMC";
    LAUNCHER_DISPLAYNAME = "PolyMC";
    LAUNCHER_COPYRIGHT = "PolyMC Contributors\n© 2012-2021 MultiMC Contributors";
    LAUNCHER_DOMAIN = "polymc.org";
    LAUNCHER_CONFIGFILE = "polymc.cfg";
    LAUNCHER_GIT = "https://github.com/PolyMC/PolyMC";
    LAUNCHER_DESKTOPFILENAME = "org.polymc.PolyMC.desktop";

    USER_AGENT = "PolyMC/6.0";
    USER_AGENT_UNCACHED = USER_AGENT + " (Uncached)";

    // Version information
    VERSION_MAJOR = 6;
    VERSION_MINOR = 0;

    BUILD_PLATFORM = "";
    BUILD_DATE = "2023-03-03";
    UPDATER_BASE = "";

    MAC_SPARKLE_PUB_KEY = "";
    MAC_SPARKLE_APPCAST_URL = "";

    if (BUILD_PLATFORM == "macOS" && !MAC_SPARKLE_PUB_KEY.isEmpty() && !MAC_SPARKLE_APPCAST_URL.isEmpty())
    {
        UPDATER_ENABLED = true;
    }

    GIT_COMMIT = "GITDIR-NOTFOUND";
    GIT_TAG = "HEAD-HASH-NOTFOUND";
    GIT_REFSPEC = "GITDIR-NOTFOUND";

    // Assume that builds outside of Git repos are "stable"
    if (GIT_REFSPEC == QStringLiteral("GITDIR-NOTFOUND")
        || GIT_TAG == QStringLiteral("GITDIR-NOTFOUND"))
    {
        GIT_REFSPEC = "refs/heads/stable";
        GIT_TAG = versionString();
    }

    if (GIT_REFSPEC.startsWith("refs/heads/"))
    {
        VERSION_CHANNEL = GIT_REFSPEC;
        VERSION_CHANNEL.remove("refs/heads/");
        if(!UPDATER_BASE.isEmpty() && !BUILD_PLATFORM.isEmpty()) {
            UPDATER_ENABLED = true;
        }
    }
    else if (!GIT_COMMIT.isEmpty())
    {
        VERSION_CHANNEL = GIT_COMMIT.mid(0, 8);
    }
    else
    {
        VERSION_CHANNEL = "unknown";
    }

    NEWS_RSS_URL = "https://polymc.org/feed/feed.xml";
    NEWS_OPEN_URL = "https://polymc.org/news";
    HELP_URL = "https://polymc.org/wiki/help-pages/%1";
    IMGUR_CLIENT_ID = "5b97b0713fba4a3";
    MSA_CLIENT_ID = "6b329578-bfec-42a3-b503-303ab3f2ac96";
    FLAME_API_KEY = "";
    FLAME_API_KEY_API_URL = "https://cf.polymc.org/api";
    META_URL = "https://meta.polymc.org/v1/";

    BUG_TRACKER_URL = "https://github.com/PolyMC/PolyMC/issues";
    TRANSLATIONS_URL = "https://hosted.weblate.org/projects/polymc/polymc/";
    MATRIX_URL = "https://matrix.to/#/#polymc:matrix.org";
    DISCORD_URL = "https://discord.gg/Z52pwxWCHP";
    SUBREDDIT_URL = "https://www.reddit.com/r/PolyMCLauncher/";
}

QString Config::versionString() const
{
    return QString("%1.%2").arg(VERSION_MAJOR).arg(VERSION_MINOR);
}

QString Config::printableVersionString() const
{
    QString vstr = versionString();

    // If the build is not a main release, append the channel
    if(VERSION_CHANNEL != "stable" && GIT_TAG != vstr)
    {
        vstr += "-" + VERSION_CHANNEL;
    }
    return vstr;
}
